using System;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.WebControls;
using AndrewConnell.TechEd.OFC431.SimulatedDelay;

namespace AndrewConnell.TechEd.OFC431 {
  public class CustomVerbsAsyncWebPart : WebPart {
    Label m_label;
    string m_callResults = string.Empty;

    /// <summary>
    /// Gets a collection of custom verbs associated with a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"></see> control.
    /// </summary>
    public override WebPartVerbCollection Verbs {
      get {
        // create an "about this session" item (client side verb)
        string jsCommand = String.Format("javascript:alert('{0}\\n\\n{1}\\n\\n{2}\\n\\n{3}\\n\\n{4}');",
              "OFC431 - Building Advanced Web Parts with ASP.NET 2.0",
              "When: Wednesday, June 6, 2007 @ 2:00p - 3:15p",
              "Where: S230 E",
              "Speaker: Andrew Connell, MVP Office SharePoint Server",
              "In this session, see what is involved with building an ASP.NET 2.0 Web Part for use in a SharePoint site, as well as the development process using SharePoint solutions. Then, take a look at some advanced topics in building Web Parts such as customizing the verbs menu, constructing connected Web Parts, and leveraging asynchronous programming techniques in Web Parts.");
        WebPartVerb aboutThisSession = new WebPartVerb("OFC431_AboutThisSession", jsCommand);
        aboutThisSession.Text = "About this session...";
        aboutThisSession.Description = "Get information about this session.";
        aboutThisSession.ImageUrl = "/_layouts/images/CNSINF16.GIF";

        // create sync task executor (server side verb)
        WebPartVerb syncCommand = new WebPartVerb("OFC431_SyncCall", OnSyncCommandClicked, "javascript:alert('This may take a moment... one moment please.');");
        syncCommand.Text = "Execute synchronous processing demo";

        // create sync task executor (server side verb)
        WebPartVerb asyncCommand = new WebPartVerb("OFC431_AsyncCall", OnAsyncCommandClicked, "javascript:alert('This may take a moment... one moment please.');");
        asyncCommand.Text = "Execute asynchronous processing demo";


        // create an array of the new verb
        WebPartVerb[] newVerbs = new WebPartVerb[] { aboutThisSession, syncCommand, asyncCommand };
        // crete verb collection to return back to .NETFX
        WebPartVerbCollection verbs = new WebPartVerbCollection(base.Verbs, newVerbs);
        return verbs;
      }
    }

    /// <summary>
    /// Called when [sync command clicked].
    /// </summary>
    protected void OnSyncCommandClicked(object sender, EventArgs e) {
      this.m_callResults = this.ExecuteSynchronousCalls();
    }

    /// <summary>
    /// Called when [async command clicked].
    /// </summary>
    protected void OnAsyncCommandClicked (object sender, EventArgs e) {
      this.m_callResults = this.ExecuteAsynchronousCalls();
    }

    /// <summary>
    /// Executes the synchronous calls.
    /// </summary>
    private string ExecuteSynchronousCalls () {
      // local to hold results
      string mockResults = string.Empty;

      // create instance of the mock web service 
      SimulatedDelay.WebService mock = new WebService();

      // call mock service 3 times
      mockResults += String.Format("Sync call 1 results: {0}<br />", mock.SimulatedDelay());
      mockResults += String.Format("Sync call 2 results: {0}<br />", mock.SimulatedDelay());
      mockResults += String.Format("Sync call 3 results: {0}<br />", mock.SimulatedDelay());

      mock = null;

      return mockResults;
    }

    /// <summary>
    /// Executes the asynchronous calls.
    /// </summary>
    private string ExecuteAsynchronousCalls () {
      // local to hold results
      string mockResults = string.Empty;

      // create three async jobs
      LongRunningJob job1 = new LongRunningJob();
      LongRunningJob job2 = new LongRunningJob();
      LongRunningJob job3 = new LongRunningJob();

      // create three page async tasks
      PageAsyncTask task1 = new PageAsyncTask(job1.OnBegin, job1.OnEnd, job1.OnTimeout, "Task1", true);
      PageAsyncTask task2 = new PageAsyncTask(job2.OnBegin, job2.OnEnd, job2.OnTimeout, "Task2", true);
      PageAsyncTask task3 = new PageAsyncTask(job3.OnBegin, job3.OnEnd, job3.OnTimeout, "Task3", true);

      // register async tasks
      Page.RegisterAsyncTask(task1);
      Page.RegisterAsyncTask(task2);
      Page.RegisterAsyncTask(task3);

      // tell page to execute all tasks
      Page.ExecuteRegisteredAsyncTasks();

      // retrieve the results
      mockResults += String.Format("Async call 1 results: {0}<br />", job1.AsyncTaskProcessResult);
      mockResults += String.Format("Async call 2 results: {0}<br />", job2.AsyncTaskProcessResult);
      mockResults += String.Format("Async call 3 results: {0}<br />", job3.AsyncTaskProcessResult);

      return mockResults;
    }
    /// <summary>
    /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
    /// </summary>
    protected override void CreateChildControls () {
      base.CreateChildControls();
      
      // add label to contain results or default instructions
      this.m_label = new Label();
      if (this.Page.IsPostBack)
        this.m_label.Text = this.m_callResults;
      else
        this.m_label.Text = "<em>Use the Web Part's verbs menu to trigger the synchronous or asynchronous test.</em>";

      this.Controls.Add(this.m_label);
    }

  }
}