using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint;
using System.Web.UI.WebControls;

namespace AndrewConnell.TechEd.OFC431 {
  public class ListConsumerWebPart : WebPart {

    ISelectedList m_connectionProvider;

    /// <summary>
    /// Connected Web Part consumer connection point.
    /// </summary>
    [ConnectionConsumer("List Consumer")]
    public void SetListDetails(ISelectedList parm) {
      this.m_connectionProvider = parm;
    }

    /// <summary>
    /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
    /// </summary>
    protected override void CreateChildControls () {
      base.CreateChildControls();

      // if no provider specified (not connected), quit...
      if (m_connectionProvider == null)
        return;
      // or if the parameters aren't set (nothing selected), quit...
      if (string.IsNullOrEmpty(m_connectionProvider.ListID) || string.IsNullOrEmpty(m_connectionProvider.WebPath))
        return;

      // grab reference to the web & list specified in the connection & bind the data
      SPGridView spGridView = SetupGridView();
      using (SPWeb spWeb = SPContext.Current.Site.OpenWeb(this.m_connectionProvider.WebPath)) {
        SPList spList = spWeb.Lists[new Guid(this.m_connectionProvider.ListID)];

        SPDataSource spDataSource = new SPDataSource();
        spDataSource.List = spList;

        spGridView.DataSource = spDataSource;
        spGridView.DataBind();

        this.Controls.Add(spGridView);
      }
    }
    /// <summary>
    /// Sets up the grid view.
    /// </summary>
    private static SPGridView SetupGridView() {
      SPGridView spGridView;
      spGridView = new SPGridView();
      spGridView.AutoGenerateColumns = false;

      BoundField colID = new BoundField();
      colID.DataField = "ID";
      colID.HeaderText = "ID";
      spGridView.Columns.Add(colID);

      BoundField colTitle = new BoundField();
      colTitle.DataField = "Title";
      colTitle.HeaderText = "Title";
      spGridView.Columns.Add(colTitle);

      return spGridView;
    }

  }
}