using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;

namespace AndrewConnell.TechEd.OFC431 {
  public class ListProviderEditorPart : EditorPart {
    DropDownList ddlSites;
    CheckBox chkIncludeSubSites;

    /// <summary>
    /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
    /// </summary>
    protected override void CreateChildControls () {
      base.CreateChildControls();

      // add site drop down
      Controls.Add(new LiteralControl("Target Site:<br/>"));
      ddlSites = new DropDownList();
      ddlSites.Width = new Unit("100%");
      PopulateSitesDropDownList(SPContext.Current.Site.RootWeb);
      Controls.Add(ddlSites);

      // add checkbox to toggle inclusion of subwebs
      Controls.Add(new LiteralControl("Include Child Sites:<br/>"));
      chkIncludeSubSites = new CheckBox();
      Controls.Add(chkIncludeSubSites);
    }
    /// <summary>
    /// Populates the sites drop down list.
    /// </summary>
    private void PopulateSitesDropDownList (SPWeb web) {
      // add passed SPWeb to the list
      ddlSites.Items.Add(new ListItem(web.ServerRelativeUrl, web.ServerRelativeUrl));

      // if there are child webs, loop through them and add as necessary
      if (web.Webs.Count>0)
        foreach (SPWeb subWeb in web.Webs) {
          PopulateSitesDropDownList(subWeb);
          subWeb.Dispose();
        }
    }
    /// <summary>
    /// Retrieves the property values from a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"></see> control for its associated <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"></see> control.
    /// </summary>
    public override void SyncChanges () {
      EnsureChildControls();

      // get reference to Web Part being edited
      ListProviderWebPart webPart = WebPartToEdit as ListProviderWebPart;

      // pull property from Web Part, setting the checkbox & drop down list selected
      chkIncludeSubSites.Checked = webPart.IncludeSubWebs;
      foreach (ListItem item in ddlSites.Items) {
        if (item.Value == webPart.WebPath) {
          item.Selected = true;
          break;
        }
      }
    }

    /// <summary>
    /// Saves the values in an <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"></see> control to the corresponding properties in the associated <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"></see> control.
    /// </summary>
    public override bool ApplyChanges () {
      EnsureChildControls();

      // get reference to Web Part being edited
      ListProviderWebPart webPart = WebPartToEdit as ListProviderWebPart;

      // save the changes from the Toolpane
      webPart.WebPath = ddlSites.SelectedValue;
      webPart.IncludeSubWebs = chkIncludeSubSites.Checked;

      return true;
    }

  }
}