using System;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using System.Collections.Generic;

namespace AndrewConnell.TechEd.OFC431 {
  public class ListProviderWebPart : WebPart, ISelectedList {

    private bool m_includeSubWebs = false;
    private string m_webPath = string.Empty;
    protected RadioButtonList m_webLists;

    /// <summary>
    /// Gets or sets a value indicating whether [include sub webs].
    /// </summary>
    public bool IncludeSubWebs {
      get {
        return m_includeSubWebs;
      }
      set {
        if (m_includeSubWebs == value)
          return;
        m_includeSubWebs = value;
      }
    }

    /// <summary>
    /// Gets or sets the web path.
    /// </summary>
    public string WebPath {
      get {
        if (string.IsNullOrEmpty(m_webPath))
          return SPContext.Current.Web.ServerRelativeUrl;
        else
          return m_webPath;
      }
      set {
        if (m_webPath == value)
          return;
        m_webPath = value;
      }
    }

    /// <summary>
    /// Gets the list ID.
    /// </summary>
    public string ListID {
      get {
        if (m_webLists == null)
          return null;

        return m_webLists.SelectedValue;
      }
    }
    /// <summary>
    /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
    /// </summary>
    protected override void CreateChildControls () {
      base.CreateChildControls();

      // create the button list
      m_webLists = new RadioButtonList();
      m_webLists.AutoPostBack = true;

      // open the web specified & add all list names to the radio button list
      using (SPWeb web = SPContext.Current.Site.OpenWeb(m_webPath)) {
        AddWebsListsToRadioButtonList(web);
      }

      Controls.Clear();
      Controls.Add(m_webLists);
    }

    /// <summary>
    /// Adds the webs lists to radio button list.
    /// </summary>
    private void AddWebsListsToRadioButtonList (SPWeb web) {
      ListItem listEntry;

      // loop through all lists and add to the radio button list
      foreach (SPList list in web.Lists) {
        if (list.OnQuickLaunch == true) {
          string listEntryName = String.Format("{0} : {1}", web.ServerRelativeUrl, list.Title);
          listEntry = new ListItem(listEntryName, list.ID.ToString());
          m_webLists.Items.Add(listEntry);
        }
      }

      // if child web's present, show them
      if (web.Webs.Count > 0 && IncludeSubWebs)
        foreach (SPWeb subWeb in web.Webs) {
          AddWebsListsToRadioButtonList(subWeb);
          subWeb.Dispose();
        }
    }

    /// <summary>
    /// Gets the selected list ID.
    /// </summary>
    [ConnectionProvider("List Picker")]
    public ISelectedList GetSelectedListID () {
      return this;
    }

    /// <summary>
    /// Returns a collection of custom <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"></see> controls that can be used to edit a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"></see> control when it is in edit mode.
    /// </summary>
    public override EditorPartCollection CreateEditorParts () {
      List<EditorPart> editorParts = new List<EditorPart>(1);

      // create instsance of custom editor part & add to collection
      EditorPart editorPart = new ListProviderEditorPart();
      editorPart.ID = ID + "_EditorPart";
      editorParts.Add(editorPart);

      return new EditorPartCollection(base.CreateEditorParts(), editorParts);
    }

  }
}