using System;

namespace AndrewConnell.TechEd.OFC431 {
  internal class LongRunningJob {

    private string m_taskProcess = string.Empty;
    protected delegate void LongRunningJobDelegate ();
    private LongRunningJobDelegate m_job;

    /// <summary>
    /// Method responsible for running the task.
    /// </summary>
    private void Execute() {
      SimulatedDelay.WebService mock = new AndrewConnell.TechEd.OFC431.SimulatedDelay.WebService();
      this.m_taskProcess = mock.SimulatedDelay();
    }

    /// <summary>
    /// Called when the async task starts.
    /// </summary>
    public IAsyncResult OnBegin(object sender, EventArgs e, AsyncCallback callback, object data) {
      this.m_job = new LongRunningJobDelegate(Execute);
      IAsyncResult result = this.m_job.BeginInvoke(callback, data);
      return result;
    }

    /// <summary>
    /// Called when the async task completes.
    /// </summary>
    public void OnEnd(IAsyncResult result) {
      this.m_job.EndInvoke(result);
    }

    /// <summary>
    /// Called when the async task times out.
    /// </summary>
    public void OnTimeout(IAsyncResult result) {
      this.m_taskProcess += "&raquo; Async task timed out.";
    }

    /// <summary>
    /// Used to return the results from the async call.
    /// </summary>
    internal string AsyncTaskProcessResult {
      get {
        return m_taskProcess;
      }
    }
  }
}